# Image Rotation Package written in pure tcl
#
# img_rotate.tcl v1.1 2001-08-17 (c) 2001 Ryan Casey
#
# Img_Rotate is free software distributed under the same license as Tcl/Tk
#
# ###############################  USAGE  #################################
#
#  For Tcl/Tk interpreters with the 'photoname -data' command (faster)
#
#  ::img_rotate::img_rotate IMAGENAME DIRECTION
#    IMAGENAME: Image name as returned by 'image create photo' command
#    DIRECTION:
#         R - Rotate Right 90 degrees
#         L - Rotate Left 90 degrees
#         U - Rotate 180 degrees
#
#  For older Tcl/Tk interpreters (8.0):
#
#  ::img_rotate::img_rotate80 IMAGENAME DIRECTION
#    IMAGENAME: Image name as returned by 'image create photo' command
#    DIRECTION:
#         R - Rotate Right 90 degrees
#         L - Rotate Left 90 degrees
#         U - Rotate 180 degrees
#
#  NOTES:
#    Currently works only with images of type photo.
#
#    Rotating an image that has tranparent parts causes those parts to turn black.
#    If anyone knows a fix for this, please send me info at scfiead@hotmail.com.
#
#    For a quick fix for older interpreters, img_rotate80 was added, but is mainly
#    a copy of img_rotate.  This will be cleaned up in a future version.
#

namespace eval img_rotate {

  proc {img_rotate} {pic direc} {
  #
    image create photo tempImage
    set width [image width $pic]
    set height [image height $pic]
    # Rotate Left
    if {$direc == "L"} {
      for {set i 0} {$i < $width} {incr i 1} {
        set curpix [$pic data -from $i 0 [expr {$i + 1}] $height] 
        tempImage put "{$curpix}" -to  0 [expr {($width - $i) - 1}]
      }
    }

    # Rotate Right
    if {$direc == "R"} {
      for {set i 0} {$i < $height} {incr i 1} {
        set curpix [$pic data -from 0 $i $width [expr {$i + 1}]] 
        tempImage put "[join $curpix]" -to [expr {($height - $i) - 1}] 0
      }
    }

    # Rotate Reverse
    if {$direc == "U"} {
      tempImage copy $pic -subsample -1 -1
    }

    $pic blank

    if {$direc == "U"} {
      $pic configure -height $height -width $width
    } else {
      $pic configure -height $width -width $height
    }

    $pic copy tempImage

    image delete tempImage

    update idletasks

    return
  }

  proc padnum {val} {
    if {$val >= 16} {
      set val [format %x $val]
    } else {
      set val "0[format %x $val]"
    }
    return $val
  }

  proc {img_rotate80} {pic direc} {
  #
    image create photo tempImage
    set width [image width $pic]
    set height [image height $pic]
    # Rotate Left
    if {$direc == "L"} {
      for {set i 0} {$i < $width} {incr i 1} {
        set curpix [list]
        for {set j $i; set k 0} {$k < $height} {incr k} {
           set datatemp [$pic get $j $k]
           lappend curpix "#[padnum [lindex $datatemp 0]][padnum [lindex $datatemp 1]][padnum [lindex $datatemp 2]]"
        }
        tempImage put "{$curpix}" -to  0 [expr {($width - $i) - 1}]
      }
    }

    # Rotate Right
    if {$direc == "R"} {
      for {set i 0} {$i < $height} {incr i 1} {
        set curpix [list]
        for {set j 0; set k $i} {$j < $width} {incr j} {
           set datatemp [$pic get $j $k]
           lappend curpix "#[padnum [lindex $datatemp 0]][padnum [lindex $datatemp 1]][padnum [lindex $datatemp 2]]"
        }
        tempImage put "[join $curpix]" -to [expr {($height - $i) - 1}] 0
      }
    }

    # Rotate Reverse
    if {$direc == "U"} {
      tempImage copy $pic -subsample -1 -1
    }

    $pic blank

    if {$direc == "U"} {
      $pic configure -height $height -width $width
    } else {
      $pic configure -height $width -width $height
    }

    $pic copy tempImage

    image delete tempImage

    update idletasks

    return
  }

}

package provide img_rotate 1.1

